Spike trains — PySpike 0.5.1 documentation

# Spike trains

In PySpike, spike trains are represented by `SpikeTrain` objects. A `SpikeTrain` object consists of the spike times given as numpy arrays as well as the edges of the spike train as `[t_start, t_end]`. The following code creates such a spike train with some arbitrary spike times:

    import numpy as np
    from pyspike import SpikeTrain

    spike_train = SpikeTrain(np.array([0.1, 0.3, 0.45, 0.6, 0.9], [0.0, 1.0]))

## Loading from text files

Typically, spike train data is loaded into PySpike from data files. The most straight-forward data files are text files where each line represents one spike train given as an sequence of spike times. An exemplary file with several spike trains is PySpike_testdata.txt. To quickly obtain spike trains from such files, PySpike provides the function `load_spike_trains_from_txt()`.

    import numpy as np
    import pyspike as spk

    spike_trains = spk.load_spike_trains_from_txt("SPIKY_testdata.txt",
                                                  edges=(0, 4000))

This function expects the name of the data file as first parameter. Furthermore, the time interval of the spike train measurement (edges of the spike trains) should be provided as a pair of start- and end-time values. Furthermore, the spike trains are sorted via `np.sort` (disable this feature by providing `is_sorted=True` as a parameter to the load function). As result, `load_spike_trains_from_txt()` returns a list of arrays containing the spike trains in the text file.

# Computing bivariate distances profiles

Important note:

> Spike trains are expected to be sorted! For performance reasons, the PySpike distance functions do not check if the spike trains provided are indeed sorted. Make sure that all your spike trains are sorted, which is ensured if you use the `load_spike_trains_from_txt()` function with the parameter is_sorted=False (default). If in doubt, use `SpikeTrain.sort()` to ensure a correctly sorted spike train.

> If you need to copy a spike train, use the `SpikeTrain.copy()` method. Simple assignment t2 = t1 does not create a copy of the spike train data, but a reference as numpy.array is used for storing the data.

## ISI-distance

The following code loads some exemplary spike trains, computes the dissimilarity profile of the ISI-distance of the first two `SpikeTrain` s, and plots it with matplotlib:

    import matplotlib.pyplot as plt
    import pyspike as spk

    spike_trains = spk.load_spike_trains_from_txt("PySpike_testdata.txt",
                                                  edges=(0, 4000))
    isi_profile = spk.isi_profile(spike_trains[0], spike_trains[1])
    x, y = isi_profile.get_plottable_data()
    plt.plot(x, y, '--k')
    print("ISI distance: %.8f" % isi_profile.avrg())
    plt.show()

The ISI-profile is a piece-wise constant function, and hence the function `isi_profile()` returns an instance of the `PieceWiseConstFunc` class. As shown above, this class allows you to obtain arrays that can be used to plot the function with `plt.plt`, but also to compute the time average, which amounts to the final scalar ISI-distance. By default, the time average is computed for the whole `PieceWiseConstFunc` function. However, it is also possible to obtain the average of a specific interval by providing a pair of floats defining the start and end of the interval. For the above example, the following code computes the ISI-distances obtained from averaging the ISI-profile over four different intervals:

    isi1 = isi_profile.avrg(interval=(0, 1000))
    isi2 = isi_profile.avrg(interval=(1000, 2000))
    isi3 = isi_profile.avrg(interval=[(0, 1000), (2000, 3000)])
    isi4 = isi_profile.avrg(interval=[(1000, 2000), (3000, 4000)])

Note, how also multiple intervals can be supplied by giving a list of tuples.

If you are only interested in the scalar ISI-distance and not the profile, you can simply use:

    isi_dist = spk.isi_distance(spike_trains[0], spike_trains[1], interval=(0, 1000))

where `interval` is optional, as above, and if omitted the ISI-distance is computed for the complete spike train.

## SPIKE-distance

To compute for the spike distance profile you use the function `spike_profile()` instead of `isi_profile` above. But the general approach is very similar:

    import matplotlib.pyplot as plt
    import pyspike as spk

    spike_trains = spk.load_spike_trains_from_txt("PySpike_testdata.txt",
                                                  edges=(0, 4000))
    spike_profile = spk.spike_profile(spike_trains[0], spike_trains[1])
    x, y = spike_profile.get_plottable_data()
    plt.plot(x, y, '--k')
    print("SPIKE distance: %.8f" % spike_profile.avrg())
    plt.show()

This short example computes and plots the SPIKE-profile of the first two spike trains in the file `PySpike_testdata.txt`.

In contrast to the ISI-profile, a SPIKE-profile is a piece-wise linear function and is therefore represented by a `PieceWiseLinFunc` object. Just like the `PieceWiseConstFunc` for the ISI-profile, the `PieceWiseLinFunc` provides a `PieceWiseLinFunc.get_plottable_data()` member function that returns arrays that can be used directly to plot the function. Furthermore, the `PieceWiseLinFunc.avrg()` member function returns the average of the profile defined as the overall SPIKE distance. As above, you can provide an interval as a pair of floats as well as a sequence of such pairs to `avrg` to specify the averaging interval if required.

Again, you can use:

    spike_dist = spk.spike_distance(spike_trains[0], spike_trains[1], interval=ival)

to compute the SPIKE distance directly, if you are not interested in the profile at all. The parameter `interval` is optional and if neglected the whole time interval is used.

## SPIKE synchronization

Important note:

> SPIKE-Synchronization measures similarity. That means, a value of zero indicates absence of synchrony, while a value of one denotes the presence of synchrony. This is exactly opposite to the other two measures: ISI- and SPIKE-distance.

SPIKE synchronization is another approach to measure spike synchrony. In contrast to the SPIKE- and ISI-distance, it measures similarity instead of dissimilarity, i.e. higher values represent larger synchrony. Another difference is that the SPIKE synchronization profile is only defined exactly at the spike times, not for the whole interval of the spike trains. Therefore, it is represented by a `DiscreteFunction`.

To compute for the spike synchronization profile, PySpike provides the function `spike_sync_profile()`. The general handling of the profile, however, is similar to the other profiles above:

    import matplotlib.pyplot as plt
    import pyspike as spk

    spike_trains = spk.load_spike_trains_from_txt("PySpike_testdata.txt",
                                                  edges=(0, 4000))
    spike_profile = spk.spike_sync_profile(spike_trains[0], spike_trains[1])
    x, y = spike_profile.get_plottable_data()

For the direct computation of the overall spike synchronization value within some interval, the `spike_sync()` function can be used:

    spike_sync = spk.spike_sync(spike_trains[0], spike_trains[1], interval=ival)

# Computing multivariate profiles and distances

To compute the multivariate ISI-profile, SPIKE-profile or SPIKE-Synchronization profile for a set of spike trains, simply provide a list of spike trains to the profile or distance functions. The following example computes the multivariate ISI-, SPIKE- and SPIKE-Sync-profile for a list of spike trains:

    spike_trains = spk.load_spike_trains_from_txt("PySpike_testdata.txt",
                                                  edges=(0, 4000))
    avrg_isi_profile = spk.isi_profile(spike_trains)
    avrg_spike_profile = spk.spike_profile(spike_trains)
    avrg_spike_sync_profile = spk.spike_sync_profile(spike_trains)

All functions also take an optional parameter `indices`, a list of indices that allows to define the spike trains that should be used for the multivariate profile. As before, if you are only interested in the distance values, and not in the profile, you can call the functions: `isi_distance()`, `spike_distance()` and `spike_sync()` with a list of spike trains. They return the scalar overall multivariate ISI-, SPIKE-distance or the SPIKE-Synchronization value.

The following code is equivalent to the bivariate example above, computing the ISI-Distance between the first two spike trains in the given interval using the `indices` parameter:

    isi_dist = spk.isi_distance(spike_trains, indices=[0, 1], interval=(0, 1000))

As you can see, the distance functions also accept an `interval` parameter that can be used to specify the begin and end of the averaging interval as a pair of floats, if neglected the complete interval is used.

Note:

> Instead of providing lists of spike trains to the profile or distance functions, you can also call those functions with many spike trains as (unnamed) parameters, e.g.:

>     # st1, st2, st3, st4 are spike trains
>     spike_prof = spk.spike_profile(st1, st2, st3, st4)

Another option to characterize large sets of spike trains are distance matrices. Each entry in the distance matrix represents a bivariate distance (similarity for SPIKE-Synchronization) of two spike trains. The distance matrix is symmetric and has zero values (ones) at the diagonal and is computed with the functions `isi_distance_matrix()`, `spike_distance_matrix()` and `spike_sync_matrix()`. The following example computes and plots the ISI- and SPIKE-distance matrix as well as the SPIKE-Synchronization-matrix, with different intervals.

    spike_trains = spk.load_spike_trains_from_txt("PySpike_testdata.txt", 4000)

    plt.figure()
    isi_distance = spk.isi_distance_matrix(spike_trains)
    plt.imshow(isi_distance, interpolation='none')
    plt.title("ISI-distance")

    plt.figure()
    spike_distance = spk.spike_distance_matrix(spike_trains, interval=(0,1000))
    plt.imshow(spike_distance, interpolation='none')
    plt.title("SPIKE-distance")

    plt.figure()
    spike_sync = spk.spike_sync_matrix(spike_trains, interval=(2000,4000))
    plt.imshow(spike_sync, interpolation='none')
    plt.title("SPIKE-Sync")

    plt.show()